@@ -46,7 +46,6 @@ static GObjectClass *bonobo_control_frame_parent_class;
 struct _BonoboControlFramePrivate {
 	BonoboControl     *inproc_control;
 	Bonobo_Control	   control;
-	guint32            plug_xid;
 	GtkWidget	  *socket;
 	Bonobo_UIContainer ui_container;
 	BonoboPropertyBag *propbag;
@@ -113,10 +112,11 @@ impl_Bonobo_ControlFrame_getAmbientProperties (PortableServer_Servant servant,
 }
 
 static Bonobo_UIContainer
-impl_Bonobo_ControlFrame_getUIHandler (PortableServer_Servant  servant,
-				       CORBA_Environment      *ev)
+impl_Bonobo_ControlFrame_getUIContainer (PortableServer_Servant  servant,
+					 CORBA_Environment      *ev)
 {
-	BonoboControlFrame *frame = BONOBO_CONTROL_FRAME (bonobo_object_from_servant (servant));
+	BonoboControlFrame *frame = BONOBO_CONTROL_FRAME (
+		bonobo_object_from_servant (servant));
 
 	if (frame->priv->ui_container == NULL)
 		return CORBA_OBJECT_NIL;
@@ -131,7 +131,8 @@ impl_Bonobo_ControlFrame_notifyActivated (PortableServer_Servant  servant,
 					  const CORBA_boolean     state,
 					  CORBA_Environment      *ev)
 {
-	BonoboControlFrame *frame = BONOBO_CONTROL_FRAME (bonobo_object_from_servant (servant));
+	BonoboControlFrame *frame = BONOBO_CONTROL_FRAME (
+		bonobo_object_from_servant (servant));
 
 	gtk_signal_emit (GTK_OBJECT (frame),
 			 control_frame_signals [ACTIVATED], state);
@@ -162,20 +163,118 @@ impl_Bonobo_ControlFrame_activateURI (PortableServer_Servant  servant,
 			 (const char *) uri, (gboolean) relative);
 }
 
+#ifdef DEBUG_CONTROL
+static void
+dump_geom (GdkWindow *window)
+{
+	gint x, y, width, height, depth;
+	
+	gdk_window_get_geometry (window, &x, &y,
+				 &width, &height, &depth);
+	
+	fprintf (stderr, "geom (%d, %d), (%d, %d), %d ",
+		 x, y, width, height, depth);
+}
+
+static void
+dump_gdk_tree (GdkWindow *window)
+{
+	GList     *l;
+	GtkWidget *widget = NULL;
+
+	gdk_window_get_user_data (window, (gpointer) &widget);
+
+	fprintf (stderr, "Window %p (parent %p) ", window,
+		 gdk_window_get_parent (window));
+
+	dump_geom (window);
+
+	if (widget) {
+		fprintf (stderr, "has widget '%s' %s ",
+			 g_type_name_from_instance ((gpointer) widget),
+			 GTK_WIDGET_VISIBLE (widget) ? "visible" : "hidden");
+	} else
+		fprintf (stderr, "No widget ");
+
+	fprintf (stderr, "gdk: %s %s ", 
+		 gdk_window_is_visible (window) ? "visible" : "invisible",
+		 gdk_window_is_viewable (window) ? "viewable" : "not viewable");
+	
+	l = gdk_window_peek_children (window);
+	fprintf (stderr, "%d children:\n", g_list_length (l));
+
+	for (; l; l = l->next)
+		dump_gdk_tree (l->data);
+
+	fprintf (stderr, "\n");
+}
+#endif
+
 void
-bonobo_control_frame_set_remote_window (BonoboControlFrame *frame)
+bonobo_control_frame_get_remote_window (BonoboControlFrame *frame,
+					CORBA_Environment  *opt_ev)
 					
 {
+	CORBA_char *id;
+	CORBA_Environment *ev, tmp_ev;
+
 	g_return_if_fail (BONOBO_IS_CONTROL_FRAME (frame));
 
+	dprintf ("bonobo_control_frame_get_remote_window "
+		 "%p %p %d %p\n", frame->priv, frame->priv->socket,
+		 GTK_WIDGET_REALIZED (frame->priv->socket),
+		 frame->priv->control);
+
 	if (!frame->priv || !frame->priv->socket ||
-	    frame->priv->plug_xid == 0)
+	    !GTK_WIDGET_REALIZED (frame->priv->socket) ||
+	    frame->priv->control == CORBA_OBJECT_NIL)
 		return;
 
-	dprintf ("gtk_socket_add_id %d\n", frame->priv->plug_xid);
+	if (!opt_ev) {
+		CORBA_exception_init (&tmp_ev);
+		ev = &tmp_ev;
+	} else
+		ev = opt_ev;
 
-	gtk_socket_add_id (GTK_SOCKET (frame->priv->socket),
-			   frame->priv->plug_xid);
+	/* Introduce ourselves to the Control. */
+	id = Bonobo_Control_getWindowId (frame->priv->control, "", ev);
+
+	if (BONOBO_EX (ev)) {
+		dprintf ("getWindowId exception\n");
+		bonobo_object_check_env (BONOBO_OBJECT (frame),
+					 frame->priv->control, ev);
+
+	} else {
+		guint32 xid;
+		BonoboPlug *plug = NULL;
+
+		xid = bonobo_control_x11_from_window_id (id);
+		dprintf ("setFrame id '%s' (=%d)\n", id, xid);
+		CORBA_free (id);
+
+		{
+			gpointer user_data = NULL;
+			if (gdk_window_lookup (xid)) {
+				gdk_window_get_user_data (gdk_window_lookup (xid),
+							  &user_data);
+				plug = user_data;
+			}
+		}
+
+		/* FIXME: What happens if we have an in-proc CORBA proxy eg.
+		 * for a remote X window ? - we need to treat these differently. */
+
+		if (plug && !frame->priv->inproc_control) {
+			g_warning ("ARGH - serious ORB screwup");
+			frame->priv->inproc_control = bonobo_plug_get_control (plug);
+		} else if (!plug && frame->priv->inproc_control) 
+			g_warning ("ARGH - different serious ORB screwup");
+
+		gtk_socket_add_id (GTK_SOCKET (frame->priv->socket), xid);
+	}		
+
+	if (!opt_ev)
+		CORBA_exception_free (ev);
 }
 
 /**
@@ -228,7 +327,7 @@ bonobo_control_frame_dispose (GObject *object)
 {
 	BonoboControlFrame *frame = BONOBO_CONTROL_FRAME (object);
 
-	dprintf ("bonobo_control_frame_dispose\n");
+	dprintf ("bonobo_control_frame_dispose %p\n", object);
 
 	if (frame->priv->socket)
 		bonobo_control_frame_set_socket (frame, NULL);
@@ -249,7 +348,7 @@ bonobo_control_frame_finalize (GObject *object)
 {
 	BonoboControlFrame *frame = BONOBO_CONTROL_FRAME (object);
 
-	dprintf ("bonobo_control_frame_finalize\n");
+	dprintf ("bonobo_control_frame_finalize %p\n", object);
 
 	g_free (frame->priv);
 	
@@ -300,12 +399,14 @@ bonobo_control_frame_class_init (BonoboControlFrameClass *klass)
 	
 	epv->getToplevelId        = impl_Bonobo_ControlFrame_getToplevelId;
 	epv->getAmbientProperties = impl_Bonobo_ControlFrame_getAmbientProperties;
-	epv->getUIHandler         = impl_Bonobo_ControlFrame_getUIHandler;
+	epv->getUIContainer       = impl_Bonobo_ControlFrame_getUIContainer;
 
 	epv->notifyActivated      = impl_Bonobo_ControlFrame_notifyActivated;
 	epv->queueResize          = impl_Bonobo_ControlFrame_queueResize;
 	epv->activateURI          = impl_Bonobo_ControlFrame_activateURI;
 	epv->unImplemented        = NULL;
+	epv->unImplemented2       = NULL;
+	epv->unImplemented3       = NULL;
 }
 
 static void
@@ -624,18 +725,14 @@ bonobo_control_frame_bind_to_control (BonoboControlFrame *frame,
 	g_object_ref (G_OBJECT (frame));
 
 	if (frame->priv->control != CORBA_OBJECT_NIL) {
-		CORBA_char *id;
-
 		if (!frame->priv->inproc_control)
 			ORBit_small_unlisten_for_broken (
 				frame->priv->control,
 				G_CALLBACK (control_connection_died_cb));
 
 		/* Unset ourselves as the frame */
-		id = Bonobo_Control_setFrame (frame->priv->control,
-					      CORBA_OBJECT_NIL, ev);
-		if (!BONOBO_EX (ev))
-			CORBA_free (id);
+		Bonobo_Control_setFrame (frame->priv->control,
+					 CORBA_OBJECT_NIL, ev);
 
 		if (frame->priv->control != CORBA_OBJECT_NIL)
 			CORBA_Object_release (frame->priv->control, ev);
@@ -647,8 +744,6 @@ bonobo_control_frame_bind_to_control (BonoboControlFrame *frame,
 		frame->priv->control = CORBA_OBJECT_NIL;
 		frame->priv->inproc_control = NULL;
 	} else {
-		CORBA_char *id;
-
 		frame->priv->control = CORBA_Object_duplicate (control, ev);
 
 		frame->priv->inproc_control = (BonoboControl *)
@@ -660,19 +755,11 @@ bonobo_control_frame_bind_to_control (BonoboControlFrame *frame,
 				G_CALLBACK (control_connection_died_cb),
 				frame, ev);
 
-		/* Introduce ourselves to the Control. */
-		id = Bonobo_Control_setFrame (control, BONOBO_OBJREF (frame), ev);
-		if (BONOBO_EX (ev))
-			bonobo_object_check_env (BONOBO_OBJECT (frame), control, ev);
-		else {
-			frame->priv->plug_xid = bonobo_control_x11_from_window_id (id);
-			dprintf ("setFrame id '%s' (=%d)\n", id, frame->priv->plug_xid);
-			CORBA_free (id);
+		Bonobo_Control_setFrame (
+			frame->priv->control,
+			BONOBO_OBJREF (frame), ev);
 
-			if (frame->priv->socket &&
-			    GTK_WIDGET_REALIZED (frame->priv->socket))
-				bonobo_control_frame_set_remote_window (frame);
-		}
+		bonobo_control_frame_get_remote_window (frame, ev);
 	}
 
 	g_object_unref (G_OBJECT (frame));
@@ -777,7 +864,7 @@ bonobo_control_frame_get_propbag (BonoboControlFrame  *frame)
  **/
 Bonobo_PropertyBag
 bonobo_control_frame_get_control_property_bag (BonoboControlFrame *frame,
-					       CORBA_Environment  *ev)
+					       CORBA_Environment  *opt_ev)
 {
 	Bonobo_PropertyBag pbag;
 	Bonobo_Control control;
@@ -786,8 +873,8 @@ bonobo_control_frame_get_control_property_bag (BonoboControlFrame *frame,
 	g_return_val_if_fail (frame != NULL, NULL);
 	g_return_val_if_fail (BONOBO_IS_CONTROL_FRAME (frame), NULL);
 
-	if (ev)
-		real_ev = ev;
+	if (opt_ev)
+		real_ev = opt_ev;
 	else {
 		CORBA_exception_init (&tmp_ev);
 		real_ev = &tmp_ev;
@@ -798,7 +885,7 @@ bonobo_control_frame_get_control_property_bag (BonoboControlFrame *frame,
 	pbag = Bonobo_Control_getProperties (control, real_ev);
 
 	if (BONOBO_EX (real_ev)) {
-		if (!ev)
+		if (!opt_ev)
 			CORBA_exception_free (&tmp_ev);
 		pbag = CORBA_OBJECT_NIL;
 	}
@@ -948,3 +1035,11 @@ bonobo_control_frame_get_socket (BonoboControlFrame *frame)
 
 	return (BonoboSocket *) frame->priv->socket;
 }
+
+BonoboUIComponent *
+bonobo_control_frame_get_popup_component (BonoboControlFrame *control_frame,
+					  CORBA_Environment  *opt_ev)
+{
+	/* FIXME: impl. me */
+	return CORBA_OBJECT_NIL;
+}
