@@ -15,12 +15,14 @@
 #include <stdlib.h>
 
 #include <gdk/gdkx.h>
+#include <gtk/gtkmenu.h>
 #include <gtk/gtksignal.h>
 
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-plug.h>
 #include <bonobo/bonobo-control.h>
 #include <bonobo/bonobo-exception.h>
+#include <bonobo/bonobo-ui-sync-menu.h>
 #include <bonobo/bonobo-control-internal.h>
 #include <bonobo/bonobo-property-bag-client.h>
 
@@ -39,21 +41,24 @@ struct _BonoboControlPrivate {
 	BonoboControlFrame  *inproc_frame;
 	BonoboUIComponent   *ui_component;
 	Bonobo_PropertyBag   propbag;
+	BonoboUIContainer   *popup_ui_container;
+	BonoboUIComponent   *popup_ui_component;
+	BonoboUISync        *popup_ui_sync;
 
 	GtkWidget           *plug;
 	GtkWidget           *widget;
 
-	gboolean             active;
-	gboolean             automerge;
+	guint                active : 1;
+	guint                automerge : 1;
 };
 
 static void
 control_frame_connection_died_cb (gpointer connection,
 				  gpointer user_data)
 {
-	BonoboControlFrame *frame = BONOBO_CONTROL_FRAME (user_data);
+	BonoboControl *control = BONOBO_CONTROL (user_data);
 
-	g_return_if_fail (frame != NULL);
+	g_return_if_fail (control != NULL);
 
 	dprintf ("The remote control frame died unexpectedly");
 }
@@ -85,14 +90,16 @@ bonobo_control_add_listener (CORBA_Object        object,
 
 /* Control lifecycle grind ... */
 void
-bonobo_control_notify_plug_died (BonoboControl *control)
+bonobo_control_notify_plug_died (BonoboControl *control,
+				 gboolean       inproc_parent_died)
 {
 	Bonobo_ControlFrame frame;
 	gboolean            end_of_life;
 
 	frame = control->priv->frame;
 
-	dprintf ("bonobo_control_notify_plug_died: ");
+	dprintf ("bonobo_control_notify_plug_died (%d): ",
+		 inproc_parent_died);
 
 	if (frame != CORBA_OBJECT_NIL) {
 		end_of_life = TRUE; /* Hack for now */
@@ -107,6 +114,7 @@ bonobo_control_notify_plug_died (BonoboControl *control)
 			dprintf ("connection broken\n");
 			break;
 		case ORBIT_CONNECTION_IN_PROC:
+			end_of_life = inproc_parent_died;
 			break;
 		}
 	} else {
@@ -131,7 +139,7 @@ bonobo_control_notify_plug_died (BonoboControl *control)
 Bonobo_Gdk_WindowId
 bonobo_control_window_id_from_x11 (guint32 x11_id)
 {
-	guchar                  str[32];
+	guchar str[32];
 
 	snprintf (str, 31, "%d", x11_id);
 	str[31] = '\0';
@@ -251,13 +259,60 @@ bonobo_control_unset_control_frame (BonoboControl     *control,
 	if (!opt_ev)
 		CORBA_exception_free (&tmp_ev);
 }
+
+static void
+create_plug (BonoboControl *control)
+{
+	GtkWidget *plug;
 	
+	plug = bonobo_plug_new (0);
+	gtk_widget_show (plug);
+	
+	bonobo_control_set_plug (control, BONOBO_PLUG (plug));
+
+	if (control->priv->widget)
+		gtk_container_add (GTK_CONTAINER (plug),
+				   control->priv->widget);
+}
+
 static CORBA_char *
+impl_Bonobo_Control_getWindowId (PortableServer_Servant servant,
+				 const CORBA_char      *cookie,
+				 CORBA_Environment     *ev)
+{
+	guint32        x11_id;
+	BonoboControl *control = BONOBO_CONTROL (
+		bonobo_object_from_servant (servant));
+
+	if (!control->priv->plug)
+		create_plug (control);
+
+	g_assert (control->priv->plug != NULL);
+
+	x11_id = gtk_plug_get_id (GTK_PLUG (control->priv->plug));
+		
+	dprintf ("plug id %d\n", x11_id);
+
+	return bonobo_control_window_id_from_x11 (x11_id);
+}
+
+static Bonobo_UIContainer
+impl_Bonobo_Control_getPopupContainer (PortableServer_Servant servant,
+				       CORBA_Environment     *ev)
+{
+	BonoboUIContainer *container;
+	
+	container = bonobo_control_get_popup_ui_container (
+		BONOBO_CONTROL (bonobo_object_from_servant (servant)));
+
+	return bonobo_object_dup_ref (BONOBO_OBJREF (container), ev);
+}
+	
+static void
 impl_Bonobo_Control_setFrame (PortableServer_Servant servant,
 			      Bonobo_ControlFrame    frame,
 			      CORBA_Environment     *ev)
 {
-	CORBA_char    *ret;
 	BonoboControl *control = BONOBO_CONTROL (
 		bonobo_object_from_servant (servant));
 
@@ -285,22 +340,7 @@ impl_Bonobo_Control_setFrame (PortableServer_Servant servant,
 		g_signal_emit (G_OBJECT (control), control_signals [SET_FRAME], 0);
 	}
 
-	if (control->priv->plug) {
-		guint32 x11_id;
-
-		x11_id = gtk_plug_get_id (GTK_PLUG (control->priv->plug));
-		
-		dprintf ("plug id %d\n", x11_id);
-
-		ret = bonobo_control_window_id_from_x11 (x11_id);
-	} else {
-		bonobo_exception_set (ev, ex_Bonobo_Control_NoContents);
-		ret = NULL;
-	}
-
 	g_object_unref (G_OBJECT (control));
-
-	return ret;
 }
 
 static void
@@ -457,16 +497,13 @@ bonobo_control_construct (BonoboControl  *control,
 	 */
 	bonobo_setup_x_error_handler ();
 
-	control->priv->widget = GTK_WIDGET (widget);
+	control->priv->widget = g_object_ref (G_OBJECT (widget));
 	gtk_container_add (GTK_CONTAINER (control->priv->plug),
 			   control->priv->widget);
 
 	control->priv->ui_component = NULL;
 	control->priv->propbag = CORBA_OBJECT_NIL;
 
-	/* Hold a ref on ourselves */
-	bonobo_object_ref (BONOBO_OBJECT (control));
-
 	return control;
 }
 
@@ -551,7 +588,7 @@ bonobo_control_dispose (GObject *object)
 {
 	BonoboControl *control = (BonoboControl *) object;
 
-	dprintf ("bonobo_control_dispose\n");
+	dprintf ("bonobo_control_dispose %p\n", object);
 
 	if (control->priv->plug)
 		bonobo_control_set_plug (control, NULL);
@@ -561,7 +598,15 @@ bonobo_control_dispose (GObject *object)
 	bonobo_control_set_ui_component    (control, NULL);
 
 	if (control->priv->widget)
-		gtk_object_destroy (GTK_OBJECT (control->priv->widget));
+		g_object_unref (G_OBJECT (control->priv->widget));
+
+	control->priv->popup_ui_container = bonobo_object_unref (
+		BONOBO_OBJECT (control->priv->popup_ui_container));
+
+	control->priv->popup_ui_component = bonobo_object_unref (
+		BONOBO_OBJECT (control->priv->popup_ui_component));
+
+	control->priv->popup_ui_sync = NULL;
 
 	bonobo_control_parent_class->dispose (object);
 }
@@ -571,7 +616,7 @@ bonobo_control_finalize (GObject *object)
 {
 	BonoboControl *control = BONOBO_CONTROL (object);
 
-	dprintf ("bonobo_control_finalize\n");
+	dprintf ("bonobo_control_finalize %p\n", object);
 
 	g_free (control->priv);
 
@@ -757,7 +802,7 @@ bonobo_control_get_remote_ui_container (BonoboControl     *control,
 	} else
 		ev = opt_ev;
 
-	ui_container = Bonobo_ControlFrame_getUIHandler (control->priv->frame, ev);
+	ui_container = Bonobo_ControlFrame_getUIContainer (control->priv->frame, ev);
 
 	bonobo_object_check_env (BONOBO_OBJECT (control), control->priv->frame, ev);
 
@@ -833,29 +878,29 @@ bonobo_control_class_init (BonoboControlClass *klass)
 
 	epv = &klass->epv;
 
-	epv->getProperties  = impl_Bonobo_Control_getProperties;
-	epv->getDesiredSize = impl_Bonobo_Control_getDesiredSize;
-	epv->setFrame       = impl_Bonobo_Control_setFrame;
-	epv->setSize        = impl_Bonobo_Control_setSize;
-	epv->setState       = impl_Bonobo_Control_setState;
-	epv->activate       = impl_Bonobo_Control_activate;
-	epv->focus          = impl_Bonobo_Control_focus;
-	epv->unImplemented  = NULL;
+	epv->getProperties     = impl_Bonobo_Control_getProperties;
+	epv->getDesiredSize    = impl_Bonobo_Control_getDesiredSize;
+	epv->getAccessible     = NULL;
+	epv->getWindowId       = impl_Bonobo_Control_getWindowId;
+	epv->getPopupContainer = impl_Bonobo_Control_getPopupContainer;
+	epv->setFrame          = impl_Bonobo_Control_setFrame;
+	epv->setSize           = impl_Bonobo_Control_setSize;
+	epv->setState          = impl_Bonobo_Control_setState;
+	epv->activate          = impl_Bonobo_Control_activate;
+	epv->focus             = impl_Bonobo_Control_focus;
+	epv->unImplemented     = NULL;
+	epv->unImplemented2    = NULL;
+	epv->unImplemented3    = NULL;
 }
 
 static void
 bonobo_control_init (BonoboControl *control)
 {
-	GtkWidget *plug;
-
 	control->priv = g_new0 (BonoboControlPrivate, 1);
 
 	control->priv->frame = CORBA_OBJECT_NIL;
 
-	plug = bonobo_plug_new (0);
-	gtk_widget_show (plug);
-
-	bonobo_control_set_plug (control, BONOBO_PLUG (plug));
+	create_plug (control);
 }
 
 BONOBO_TYPE_FUNC_FULL (BonoboControl, 
@@ -1150,3 +1195,98 @@ bonobo_control_get_plug (BonoboControl *control)
 
 	return (BonoboPlug *) control->priv->plug;
 }
+
+void
+bonobo_control_set_popup_ui_container (BonoboControl     *control,
+				       BonoboUIContainer *ui_container)
+{
+	g_return_if_fail (BONOBO_IS_CONTROL (control));
+	g_return_if_fail (BONOBO_IS_UI_CONTAINER (ui_container));
+
+	g_assert (control->priv->popup_ui_container == NULL);
+
+	control->priv->popup_ui_container = bonobo_object_ref (
+		BONOBO_OBJECT (ui_container));
+}
+
+BonoboUIContainer *
+bonobo_control_get_popup_ui_container (BonoboControl *control)
+{
+	g_return_val_if_fail (BONOBO_IS_CONTROL (control), NULL);
+
+	if (!control->priv->popup_ui_container) {
+		BonoboUIEngine *engine;
+
+		engine = bonobo_ui_engine_new (G_OBJECT (control));
+
+		control->priv->popup_ui_sync = bonobo_ui_sync_menu_new (
+			engine, NULL, NULL, NULL);
+
+		bonobo_ui_engine_add_sync (
+			engine, control->priv->popup_ui_sync);
+
+		control->priv->popup_ui_container = bonobo_ui_container_new ();
+		bonobo_ui_container_set_engine (
+			control->priv->popup_ui_container, engine);
+
+		g_signal_connect_closure (
+			G_OBJECT (control->priv->popup_ui_container),
+			"destroy",
+			g_cclosure_new_swap (
+				G_CALLBACK (g_object_unref), engine, NULL), 0);
+	}
+
+	return control->priv->popup_ui_container;
+}
+
+BonoboUIComponent *
+bonobo_control_get_popup_ui_component (BonoboControl *control)
+{
+	BonoboUIContainer *ui_container;
+
+	g_return_val_if_fail (BONOBO_IS_CONTROL (control), NULL);
+
+	if (!control->priv->popup_ui_component) {
+		ui_container = bonobo_control_get_popup_ui_container (control);
+
+		control->priv->popup_ui_component = 
+			bonobo_ui_component_new_default ();
+
+		bonobo_ui_component_set_container (
+			control->priv->popup_ui_component,
+			BONOBO_OBJREF (ui_container), NULL);
+	}
+
+	return control->priv->popup_ui_component;
+}
+
+gboolean
+bonobo_control_do_popup (BonoboControl *control,
+			 guint          button,
+			 guint32        activate_time)
+{
+	char      *path;
+	GtkWidget *menu;
+
+	g_return_val_if_fail (BONOBO_IS_CONTROL (control), FALSE);
+
+	if (!control->priv->popup_ui_container)
+		return FALSE;
+
+	path = g_strdup_printf ("/popups/button%d", button);
+
+	menu = gtk_menu_new ();
+
+	bonobo_ui_sync_menu_add_popup (
+		BONOBO_UI_SYNC_MENU (control->priv->popup_ui_sync),
+		GTK_MENU (menu), path);
+
+	g_free (path);
+
+	gtk_widget_show (menu);
+
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
+			NULL, NULL, button, activate_time);
+
+	return TRUE;
+}
