@@ -16,7 +16,6 @@
 #include <bonobo/bonobo-ui-component.h>
 #include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-ui-marshal.h>
-#include <bonobo/bonobo-types.h>
 #include <libxml/tree.h>
 #include <libxml/parser.h>
 
@@ -585,6 +584,8 @@ bonobo_ui_component_set (BonoboUIComponent  *component,
 			 const char         *xml,
 			 CORBA_Environment  *opt_ev)
 {
+	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
+
 	GET_CLASS (component)->xml_set (component, path, xml, opt_ev);
 }
 
@@ -598,7 +599,6 @@ impl_xml_set (BonoboUIComponent  *component,
 	Bonobo_UIContainer container;
 	char              *name;
 
-	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
 	container = component->priv->container;
 	g_return_if_fail (container != CORBA_OBJECT_NIL);
 
@@ -705,6 +705,8 @@ bonobo_ui_component_get (BonoboUIComponent *component,
 			 gboolean           recurse,
 			 CORBA_Environment *opt_ev)
 {
+	g_return_val_if_fail (BONOBO_IS_UI_COMPONENT (component), NULL);
+
 	return GET_CLASS (component)->xml_get (component, path, recurse, opt_ev);
 }
 
@@ -718,7 +720,6 @@ impl_xml_get (BonoboUIComponent *component,
 	CORBA_char *xml;
 	Bonobo_UIContainer container;
 
-	g_return_val_if_fail (BONOBO_IS_UI_COMPONENT (component), NULL);
 	container = component->priv->container;
 	g_return_val_if_fail (container != CORBA_OBJECT_NIL, NULL);
 
@@ -800,6 +801,8 @@ bonobo_ui_component_rm (BonoboUIComponent  *component,
 			const char         *path,
 			CORBA_Environment  *ev)
 {
+	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
+
 	GET_CLASS (component)->xml_rm (component, path, ev);
 }
 
@@ -812,7 +815,6 @@ impl_xml_rm (BonoboUIComponent  *component,
 	CORBA_Environment *real_ev, tmp_ev;
 	Bonobo_UIContainer container;
 
-	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
 	container = component->priv->container;
 	g_return_if_fail (container != CORBA_OBJECT_NIL);
 
@@ -987,6 +989,8 @@ void
 bonobo_ui_component_freeze (BonoboUIComponent *component,
 			    CORBA_Environment *ev)
 {
+	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
+
 	GET_CLASS (component)->freeze (component, ev);
 }
 
@@ -997,7 +1001,6 @@ impl_freeze (BonoboUIComponent *component,
 	CORBA_Environment *real_ev, tmp_ev;
 	Bonobo_UIContainer container;
 
-	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
 	container = component->priv->container;
 	g_return_if_fail (container != CORBA_OBJECT_NIL);
 
@@ -1034,6 +1037,8 @@ void
 bonobo_ui_component_thaw (BonoboUIComponent *component,
 			  CORBA_Environment *ev)
 {
+	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
+
 	GET_CLASS (component)->thaw (component, ev);
 }
 
@@ -1044,7 +1049,6 @@ impl_thaw (BonoboUIComponent *component,
 	CORBA_Environment *real_ev, tmp_ev;
 	Bonobo_UIContainer container;
 
-	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
 	container = component->priv->container;
 	g_return_if_fail (container != CORBA_OBJECT_NIL);
 
@@ -1085,6 +1089,8 @@ bonobo_ui_component_set_prop (BonoboUIComponent  *component,
 			      const char         *value,
 			      CORBA_Environment  *opt_ev)
 {
+	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
+
 	GET_CLASS (component)->set_prop (component, path, prop, value, opt_ev);
 }
 
@@ -1101,7 +1107,6 @@ impl_set_prop (BonoboUIComponent  *component,
 	g_return_if_fail (path != NULL);
 	g_return_if_fail (prop != NULL);
 	g_return_if_fail (value != NULL);
-	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
 
 	container = component->priv->container;
 	g_return_if_fail (container != CORBA_OBJECT_NIL);
@@ -1140,6 +1145,8 @@ bonobo_ui_component_get_prop (BonoboUIComponent *component,
 			      const char        *prop,
 			      CORBA_Environment *opt_ev)
 {
+	g_return_val_if_fail (BONOBO_IS_UI_COMPONENT (component), NULL);
+
 	return GET_CLASS (component)->get_prop (component, path, prop, opt_ev);
 }
 
@@ -1150,29 +1157,29 @@ impl_get_prop (BonoboUIComponent *component,
 	       CORBA_Environment *opt_ev)
 {
 	Bonobo_UIContainer container;
-	CORBA_Environment *real_ev, tmp_ev;
+	CORBA_Environment *ev, tmp_ev;
 	CORBA_char        *ret;
 	gchar             *retval;
 
 	g_return_val_if_fail (path != NULL, NULL);
 	g_return_val_if_fail (prop != NULL, NULL);
-	g_return_val_if_fail (BONOBO_IS_UI_COMPONENT (component), NULL);
 
 	container = component->priv->container;
 	g_return_val_if_fail (container != CORBA_OBJECT_NIL, NULL);
 
 	if (opt_ev)
-		real_ev = opt_ev;
+		ev = opt_ev;
 	else {
 		CORBA_exception_init (&tmp_ev);
-		real_ev = &tmp_ev;
+		ev = &tmp_ev;
 	}
 
 	ret = Bonobo_UIContainer_getAttr (
-		container, path, prop, real_ev);
+		container, path, prop, ev);
 
-	if (BONOBO_EX (real_ev)) {
-		if (!opt_ev)
+	if (BONOBO_EX (ev)) {
+		if (!opt_ev && strcmp (BONOBO_EX_REPOID (ev),
+				       ex_Bonobo_UIContainer_NonExistentAttr))
 			g_warning ("Invalid path '%s' on prop '%s' get",
 				   path, prop);
 		ret = NULL;
@@ -1203,39 +1210,40 @@ bonobo_ui_component_path_exists (BonoboUIComponent *component,
 				 const char        *path,
 				 CORBA_Environment *ev)
 {
+	g_return_val_if_fail (BONOBO_IS_UI_COMPONENT (component), FALSE);
+
 	return GET_CLASS (component)->exists (component, path, ev);
 }
 
 static gboolean
 impl_exists (BonoboUIComponent *component,
 	     const char        *path,
-	     CORBA_Environment *ev)
+	     CORBA_Environment *opt_ev)
 {
 	gboolean ret;
 	Bonobo_UIContainer container;
-	CORBA_Environment *real_ev, tmp_ev;
+	CORBA_Environment *ev, tmp_ev;
 
-	g_return_val_if_fail (BONOBO_IS_UI_COMPONENT (component), FALSE);
 	container = component->priv->container;
 	g_return_val_if_fail (container != CORBA_OBJECT_NIL, FALSE);
 
-	if (ev)
-		real_ev = ev;
+	if (opt_ev)
+		ev = opt_ev;
 	else {
 		CORBA_exception_init (&tmp_ev);
-		real_ev = &tmp_ev;
+		ev = &tmp_ev;
 	}
 
-	ret = Bonobo_UIContainer_exists (container, path, real_ev);
+	ret = Bonobo_UIContainer_exists (container, path, ev);
 
-	if (BONOBO_EX (real_ev)) {
+	if (BONOBO_EX (ev)) {
 		ret = FALSE;
-		if (!ev)
+		if (!opt_ev)
 			g_warning ("Serious exception on path_exists '$%s'",
-				   bonobo_exception_get_text (real_ev));
+				   bonobo_exception_get_text (ev));
 	}
 
-	if (!ev)
+	if (!opt_ev)
 		CORBA_exception_free (&tmp_ev);
 
 	return ret;
@@ -1284,11 +1292,16 @@ void
 bonobo_ui_component_unset_container (BonoboUIComponent *component,
 				     CORBA_Environment *opt_ev)
 {
+	Bonobo_UIContainer container;
+
 	g_return_if_fail (BONOBO_IS_UI_COMPONENT (component));
 
 	bonobo_object_ref (BONOBO_OBJECT (component));
 
-	if (component->priv->container != CORBA_OBJECT_NIL) {
+	container = component->priv->container;
+	component->priv->container = CORBA_OBJECT_NIL;
+
+	if (container != CORBA_OBJECT_NIL) {
 		CORBA_Environment *ev, temp_ev;
 		char              *name;
 		
@@ -1298,12 +1311,10 @@ bonobo_ui_component_unset_container (BonoboUIComponent *component,
 		} else
 			ev = opt_ev;
 
-		bonobo_ui_component_rm (component, "/", ev);
-
 		name = component->priv->name ? component->priv->name : "";
 
-		Bonobo_UIContainer_deregisterComponent (
-			component->priv->container, name, ev);
+		Bonobo_UIContainer_removeNode (container, "/", name, ev);
+		Bonobo_UIContainer_deregisterComponent (container, name, ev);
 		
 		if (!opt_ev && BONOBO_EX (ev)) {
 			char *err;
@@ -1312,14 +1323,12 @@ bonobo_ui_component_unset_container (BonoboUIComponent *component,
 			g_free (err);
 		}
 
-		bonobo_object_release_unref (component->priv->container, ev);
+		bonobo_object_release_unref (container, ev);
 
 		if (!opt_ev)
 			CORBA_exception_free (&temp_ev);
 	}
 
-	component->priv->container = CORBA_OBJECT_NIL;
-
 	bonobo_object_unref (BONOBO_OBJECT (component));
 }
 
